@@ -1,7 +1,6 @@
 /* -*- mode: js; js-basic-offset: 4; indent-tabs-mode: nil -*- */
 
 const Clutter = imports.gi.Clutter;
-const Lang = imports.lang;
 const Meta = imports.gi.Meta;
 const Shell = imports.gi.Shell;
 
@@ -23,11 +22,9 @@ function enable() {
     AltTab.WindowSwitcherPopup.prototype._keyPressHandler = function(keysym, action) {
         switch(action) {
             case Meta.KeyBindingAction.SWITCH_APPLICATIONS:
-            case Meta.KeyBindingAction.SWITCH_GROUP:
               action = Meta.KeyBindingAction.SWITCH_WINDOWS;
               break;
             case Meta.KeyBindingAction.SWITCH_APPLICATIONS_BACKWARD:
-            case Meta.KeyBindingAction.SWITCH_GROUP_BACKWARD:
               action = Meta.KeyBindingAction.SWITCH_WINDOWS_BACKWARD;
               break;
         }
@@ -45,19 +42,19 @@ function enable() {
             tabPopup.destroy();
     };
 
-    setKeybinding('switch-applications', Lang.bind(Main.wm, Main.wm._forcedWindowSwitcher));
-    setKeybinding('switch-group', Lang.bind(Main.wm, Main.wm._forcedWindowSwitcher));
-    setKeybinding('switch-applications-backward', Lang.bind(Main.wm, Main.wm._forcedWindowSwitcher));
-    setKeybinding('switch-group-backward', Lang.bind(Main.wm, Main.wm._forcedWindowSwitcher));
+    setKeybinding('switch-applications',
+                  Main.wm._forcedWindowSwitcher.bind(Main.wm));
+    setKeybinding('switch-applications-backward',
+                  Main.wm._forcedWindowSwitcher.bind(Main.wm));
 }
 
 function disable() {
     var prop;
 
-    setKeybinding('switch-applications', Lang.bind(Main.wm, Main.wm._startSwitcher));
-    setKeybinding('switch-group', Lang.bind(Main.wm, Main.wm._startSwitcher));
-    setKeybinding('switch-applications-backward', Lang.bind(Main.wm, Main.wm._startSwitcher));
-    setKeybinding('switch-group-backward', Lang.bind(Main.wm, Main.wm._startSwitcher));
+    setKeybinding('switch-applications',
+                  Main.wm._startSwitcher.bind(Main.wm));
+    setKeybinding('switch-applications-backward',
+                  Main.wm._startSwitcher.bind(Main.wm));
 
     for (prop in injections)
         AltTab.WindowSwitcherPopup.prototype[prop] = injections[prop];
