@@ -3,7 +3,6 @@
 const GLib = imports.gi.GLib;
 const Gio = imports.gi.Gio;
 const Shell = imports.gi.Shell;
-const Lang = imports.lang;
 const Mainloop = imports.mainloop;
 const Signals = imports.signals;
 const St = imports.gi.St;
@@ -17,7 +16,7 @@ const Util = imports.misc.util;
 
 const Gettext = imports.gettext.domain('gnome-shell-extensions');
 const _ = Gettext.gettext;
-const N_ = function(x) { return x; }
+const N_ = x => x;
 
 const BACKGROUND_SCHEMA = 'org.gnome.desktop.background';
 
@@ -28,24 +27,26 @@ const Hostname1Iface = '<node> \
 </node>';
 const Hostname1 = Gio.DBusProxy.makeProxyWrapper(Hostname1Iface);
 
-const PlaceInfo = new Lang.Class({
-    Name: 'PlaceInfo',
+class PlaceInfo {
+    constructor() {
+        this._init.apply(this, arguments);
+    }
 
-    _init: function(kind, file, name, icon) {
+    _init(kind, file, name, icon) {
         this.kind = kind;
         this.file = file;
         this.name = name || this._getFileName();
         this.icon = icon ? new Gio.ThemedIcon({ name: icon }) : this.getIcon();
-    },
+    }
 
-    destroy: function() {
-    },
+    destroy() {
+    }
 
-    isRemovable: function() {
+    isRemovable() {
         return false;
-    },
+    }
 
-    _createLaunchCallback: function(launchContext, tryMount) {
+    _createLaunchCallback(launchContext, tryMount) {
         return (_ignored, result) => {
             try {
                 Gio.AppInfo.launch_default_for_uri_finish(result);
@@ -78,20 +79,20 @@ const PlaceInfo = new Lang.Class({
                 Main.notifyError(_("Failed to launch “%s”").format(this.name), e.message);
             }
         }
-    },
+    }
 
-    launch: function(timestamp) {
+    launch(timestamp) {
         let launchContext = global.create_app_launch_context(timestamp, -1);
         let callback = this._createLaunchCallback(launchContext, true);
         Gio.AppInfo.launch_default_for_uri_async(this.file.get_uri(),
                                                  launchContext,
                                                  null,
                                                  callback);
-    },
+    }
 
-    getIcon: function() {
+    getIcon() {
         this.file.query_info_async('standard::symbolic-icon', 0, 0, null,
-                                   Lang.bind(this, function(file, result) {
+                                   (file, result) => {
                                        try {
                                            let info = file.query_info_finish(result);
                                            this.icon = info.get_symbolic_icon();
@@ -99,7 +100,7 @@ const PlaceInfo = new Lang.Class({
                                        } catch(e if e instanceof Gio.IOErrorEnum) {
                                            return;
                                        }
-                                   }));
+                                   });
 
         // return a generic icon for this kind for now, until we have the
         // icon from the query info above
@@ -116,101 +117,135 @@ const PlaceInfo = new Lang.Class({
             else
                 return new Gio.ThemedIcon({ name: 'folder-symbolic' });
         }
-    },
+    }
 
-    _getFileName: function() {
+    _getFileName() {
         try {
             let info = this.file.query_info('standard::display-name', 0, null);
             return info.get_display_name();
         } catch(e if e instanceof Gio.IOErrorEnum) {
             return this.file.get_basename();
         }
-    },
-});
+    }
+};
 Signals.addSignalMethods(PlaceInfo.prototype);
 
-const RootInfo = new Lang.Class({
-    Name: 'RootInfo',
-    Extends: PlaceInfo,
+class RootInfo extends PlaceInfo {
+    _init() {
+        super._init('devices', Gio.File.new_for_path('/'), _("Computer"));
 
-    _init: function() {
-        this.parent('devices', Gio.File.new_for_path('/'), _("Computer"));
+        let busName = 'org.freedesktop.hostname1';
+        let objPath = '/org/freedesktop/hostname1';
+        new Hostname1(Gio.DBus.system, busName, objPath, (obj, error) => {
+            if (error)
+                return;
 
-        this._proxy = new Hostname1(Gio.DBus.system,
-                                    'org.freedesktop.hostname1',
-                                    '/org/freedesktop/hostname1',
-                                    Lang.bind(this, function(obj, error) {
-                                        if (error)
-                                            return;
+            this._proxy = obj;
+            this._proxy.connect('g-properties-changed',
+                                this._propertiesChanged.bind(this));
+            this._propertiesChanged(obj);
+        });
+    }
 
-                                        this._proxy.connect('g-properties-changed',
-                                                            Lang.bind(this, this._propertiesChanged));
-                                        this._propertiesChanged(obj);
-                                    }));
-    },
-
-    getIcon: function() {
+    getIcon() {
         return new Gio.ThemedIcon({ name: 'drive-harddisk-symbolic' });
-    },
+    }
 
-    _propertiesChanged: function(proxy) {
+    _propertiesChanged(proxy) {
         // GDBusProxy will emit a g-properties-changed when hostname1 goes down
         // ignore it
         if (proxy.g_name_owner) {
             this.name = proxy.PrettyHostname || _("Computer");
             this.emit('changed');
         }
-    },
-
-    destroy: function() {
-        this._proxy.run_dispose();
-        this.parent();
     }
-});
+
+    destroy() {
+        if (this._proxy) {
+            this._proxy.run_dispose();
+            this._proxy = null;
+        }
+        super.destroy();
+    }
+};
 
 
-const PlaceDeviceInfo = new Lang.Class({
-    Name: 'PlaceDeviceInfo',
-    Extends: PlaceInfo,
-
-    _init: function(kind, mount) {
+class PlaceDeviceInfo extends PlaceInfo {
+    _init(kind, mount) {
         this._mount = mount;
-        this.parent(kind, mount.get_root(), mount.get_name());
-    },
+        super._init(kind, mount.get_root(), mount.get_name());
+    }
 
-    getIcon: function() {
+    getIcon() {
         return this._mount.get_symbolic_icon();
     }
-});
 
-const PlaceVolumeInfo = new Lang.Class({
-    Name: 'PlaceVolumeInfo',
-    Extends: PlaceInfo,
+    isRemovable() {
+        return this._mount.can_eject();
+    }
 
-    _init: function(kind, volume) {
+    eject() {
+        let mountOp = new ShellMountOperation.ShellMountOperation(this._mount);
+
+        if (this._mount.can_eject())
+            this._mount.eject_with_operation(Gio.MountUnmountFlags.NONE,
+                                             mountOp.mountOp,
+                                             null, // Gio.Cancellable
+                                             this._ejectFinish.bind(this));
+        else
+            this._mount.unmount_with_operation(Gio.MountUnmountFlags.NONE,
+                                               mountOp.mountOp,
+                                               null, // Gio.Cancellable
+                                               this._unmountFinish.bind(this));
+    }
+
+    _ejectFinish(mount, result) {
+        try {
+            mount.eject_with_operation_finish(result);
+        } catch(e) {
+            this._reportFailure(e);
+        }
+    }
+
+    _unmountFinish(mount, result) {
+        try {
+            mount.unmount_with_operation_finish(result);
+        } catch(e) {
+            this._reportFailure(e);
+        }
+    }
+
+    _reportFailure(exception) {
+        let msg = _("Ejecting drive “%s” failed:").format(this._mount.get_name());
+        Main.notifyError(msg, exception.message);
+    }
+};
+
+class PlaceVolumeInfo extends PlaceInfo {
+    _init(kind, volume) {
         this._volume = volume;
-        this.parent(kind, volume.get_activation_root(), volume.get_name());
-    },
+        super._init(kind, volume.get_activation_root(), volume.get_name());
+    }
 
-    launch: function(timestamp) {
+    launch(timestamp) {
         if (this.file) {
-            this.parent(timestamp);
+            super.launch(timestamp);
             return;
         }
 
-        this._volume.mount(0, null, null, Lang.bind(this, function(volume, result) {
+        this._volume.mount(0, null, null, (volume, result) => {
             volume.mount_finish(result);
 
             let mount = volume.get_mount();
             this.file = mount.get_root();
-            this.parent(timestamp);
-        }));
-    },
+            super.launch(timestamp);
+        });
+    }
 
-    getIcon: function() {
+    getIcon() {
         return this._volume.get_symbolic_icon();
     }
-});
+};
 
 const DEFAULT_DIRECTORIES = [
     GLib.UserDirectory.DIRECTORY_DOCUMENTS,
@@ -220,10 +255,8 @@ const DEFAULT_DIRECTORIES = [
     GLib.UserDirectory.DIRECTORY_VIDEOS,
 ];
 
-var PlacesManager = new Lang.Class({
-    Name: 'PlacesManager',
-
-    _init: function() {
+var PlacesManager = class {
+    constructor() {
         this._places = {
             special: [],
             devices: [],
@@ -234,7 +267,7 @@ var PlacesManager = new Lang.Class({
         this._settings = new Gio.Settings({ schema_id: BACKGROUND_SCHEMA });
         this._showDesktopIconsChangedId =
             this._settings.connect('changed::show-desktop-icons',
-                                   Lang.bind(this, this._updateSpecials));
+                                   this._updateSpecials.bind(this));
         this._updateSpecials();
 
         /*
@@ -250,35 +283,35 @@ var PlacesManager = new Lang.Class({
 
         if (this._bookmarksFile) {
             this._monitor = this._bookmarksFile.monitor_file(Gio.FileMonitorFlags.NONE, null);
-            this._monitor.connect('changed', Lang.bind(this, function () {
+            this._monitor.connect('changed', () => {
                 if (this._bookmarkTimeoutId > 0)
                     return;
                 /* Defensive event compression */
-                this._bookmarkTimeoutId = Mainloop.timeout_add(100, Lang.bind(this, function () {
+                this._bookmarkTimeoutId = Mainloop.timeout_add(100, () => {
                     this._bookmarkTimeoutId = 0;
                     this._reloadBookmarks();
                     return false;
-                }));
-            }));
+                });
+            });
 
             this._reloadBookmarks();
         }
-    },
+    }
 
-    _connectVolumeMonitorSignals: function() {
+    _connectVolumeMonitorSignals() {
         const signals = ['volume-added', 'volume-removed', 'volume-changed',
                          'mount-added', 'mount-removed', 'mount-changed',
                          'drive-connected', 'drive-disconnected', 'drive-changed'];
 
         this._volumeMonitorSignals = [];
-        let func = Lang.bind(this, this._updateMounts);
+        let func = this._updateMounts.bind(this);
         for (let i = 0; i < signals.length; i++) {
             let id = this._volumeMonitor.connect(signals[i], func);
             this._volumeMonitorSignals.push(id);
         }
-    },
+    }
 
-    destroy: function() {
+    destroy() {
         if (this._settings)
             this._settings.disconnect(this._showDesktopIconsChangedId);
         this._settings = null;
@@ -290,10 +323,10 @@ var PlacesManager = new Lang.Class({
             this._monitor.cancel();
         if (this._bookmarkTimeoutId)
             Mainloop.source_remove(this._bookmarkTimeoutId);
-    },
+    }
 
-    _updateSpecials: function() {
-        this._places.special.forEach(function (p) { p.destroy(); });
+    _updateSpecials() {
+        this._places.special.forEach(p => { p.destroy(); });
         this._places.special = [];
 
         let homePath = GLib.get_home_dir();
@@ -323,21 +356,19 @@ var PlacesManager = new Lang.Class({
             specials.push(info);
         }
 
-        specials.sort(function(a, b) {
-            return GLib.utf8_collate(a.name, b.name);
-        });
+        specials.sort((a, b) => GLib.utf8_collate(a.name, b.name));
         this._places.special = this._places.special.concat(specials);
 
         this.emit('special-updated');
-    },
+    }
 
-    _updateMounts: function() {
+    _updateMounts() {
         let networkMounts = [];
         let networkVolumes = [];
 
-        this._places.devices.forEach(function (p) { p.destroy(); });
+        this._places.devices.forEach(p => { p.destroy(); });
         this._places.devices = [];
-        this._places.network.forEach(function (p) { p.destroy(); });
+        this._places.network.forEach(p => { p.destroy(); });
         this._places.network = [];
 
         /* Add standard places */
@@ -412,9 +443,9 @@ var PlacesManager = new Lang.Class({
 
         this.emit('devices-updated');
         this.emit('network-updated');
-    },
+    }
 
-    _findBookmarksFile: function() {
+    _findBookmarksFile() {
         let paths = [
             GLib.build_filenamev([GLib.get_user_config_dir(), 'gtk-3.0', 'bookmarks']),
             GLib.build_filenamev([GLib.get_home_dir(), '.gtk-bookmarks']),
@@ -426,9 +457,9 @@ var PlacesManager = new Lang.Class({
         }
 
         return null;
-    },
+    }
 
-    _reloadBookmarks: function() {
+    _reloadBookmarks() {
 
         this._bookmarks = [];
 
@@ -476,9 +507,9 @@ var PlacesManager = new Lang.Class({
         this._places.bookmarks = bookmarks;
 
         this.emit('bookmarks-updated');
-    },
+    }
 
-    _addMount: function(kind, mount) {
+    _addMount(kind, mount) {
         let devItem;
 
         try {
@@ -488,9 +519,9 @@ var PlacesManager = new Lang.Class({
         }
 
         this._places[kind].push(devItem);
-    },
+    }
 
-    _addVolume: function(kind, volume) {
+    _addVolume(kind, volume) {
         let volItem;
 
         try {
@@ -500,10 +531,10 @@ var PlacesManager = new Lang.Class({
         }
 
         this._places[kind].push(volItem);
-    },
+    }
 
-    get: function (kind) {
+    get(kind) {
         return this._places[kind];
     }
-});
+};
 Signals.addSignalMethods(PlacesManager.prototype);
