@@ -1,8 +1,8 @@
 // -*- mode: js2; indent-tabs-mode: nil; js2-basic-offset: 4 -*-
 
+const Gio = imports.gi.Gio;
 const GObject = imports.gi.GObject;
 const Gtk = imports.gi.Gtk;
-const Lang = imports.lang;
 
 const Gettext = imports.gettext.domain('gnome-shell-extensions');
 const _ = Gettext.gettext;
@@ -16,28 +16,25 @@ function init() {
     Convenience.initTranslations();
 }
 
-const WindowListPrefsWidget = new GObject.Class({
-    Name: 'WindowList.Prefs.Widget',
-    GTypeName: 'WindowListPrefsWidget',
-    Extends: Gtk.Frame,
+const WindowListPrefsWidget = GObject.registerClass(
+class WindowListPrefsWidget extends Gtk.Grid {
+    _init(params) {
+        super._init(params);
 
-    _init: function(params) {
-        this.parent(params);
-
-        this.shadow_type = Gtk.ShadowType.NONE;
         this.margin = 24;
+        this.row_spacing = 6;
+        this.orientation = Gtk.Orientation.VERTICAL;
 
-        let title = '<b>' + _("Window Grouping") + '</b>';
-        let titleLabel = new Gtk.Label({ use_markup: true, label: title });
-        this.set_label_widget(titleLabel);
+        let groupingLabel = '<b>' + _("Window Grouping") + '</b>';
+        this.add(new Gtk.Label({ label: groupingLabel, use_markup: true,
+                                 halign: Gtk.Align.START }));
 
         let align = new Gtk.Alignment({ left_padding: 12 });
         this.add(align);
 
         let grid = new Gtk.Grid({ orientation: Gtk.Orientation.VERTICAL,
                                   row_spacing: 6,
-                                  column_spacing: 6,
-                                  margin_top: 6 });
+                                  column_spacing: 6 });
         align.add(grid);
 
         this._settings = Convenience.getSettings();
@@ -65,11 +62,16 @@ const WindowListPrefsWidget = new GObject.Class({
                                           group: radio });
             grid.add(radio);
 
-            radio.connect('toggled', Lang.bind(this, function(button) {
+            radio.connect('toggled', button => {
                 if (button.active)
                     this._settings.set_string('grouping-mode', mode);
-            }));
+            });
         }
+
+        let check = new Gtk.CheckButton({ label: _("Show on all monitors"),
+                                          margin_top: 6 });
+        this._settings.bind('show-on-all-monitors', check, 'active', Gio.SettingsBindFlags.DEFAULT);
+        this.add(check);
     }
 });
 
