@@ -1,6 +1,5 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 const Clutter = imports.gi.Clutter;
-const Lang = imports.lang;
 const Mainloop = imports.mainloop;
 const St = imports.gi.St;
 
@@ -233,9 +232,9 @@ function enable() {
                                                           visible: false }));
 
             this.actor.add_actor(this._tip);
-            let signalId = this.actor.connect('notify::scale-x', Lang.bind(this, function() {
+            let signalId = this.actor.connect('notify::scale-x', () => {
                 this._tip.set_scale(1 / this.actor.scale_x, 1 / this.actor.scale_x);
-            }));
+            });
             connectedSignals.push({ obj: this.actor, id: signalId });
         } else
             this._tip = null;
@@ -244,8 +243,10 @@ function enable() {
     workViewInjections['_init'] = injectToFunction(WorkspacesView.WorkspacesView.prototype, '_init', function(width, height, x, y, workspaces) {
         this._pickWorkspace = false;
         this._pickWindow = false;
-        this._keyPressEventId = global.stage.connect('key-press-event', Lang.bind(this, this._onKeyPress));
-        this._keyReleaseEventId = global.stage.connect('key-release-event', Lang.bind(this, this._onKeyRelease));
+        this._keyPressEventId =
+            global.stage.connect('key-press-event', this._onKeyPress.bind(this));
+        this._keyReleaseEventId =
+            global.stage.connect('key-release-event', this._onKeyRelease.bind(this));
         connectedSignals.push({ obj: global.stage, id: this._keyPressEventId });
         connectedSignals.push({ obj: global.stage, id: this._keyReleaseEventId });
     });
@@ -274,10 +275,10 @@ function disable() {
     for (i in workViewInjections)
         removeInjection(WorkspacesView.WorkspacesView.prototype, workViewInjections, i);
 
-    for each (i in connectedSignals)
+    for (i of connectedSignals)
         i.obj.disconnect(i.id);
 
-    for each (i in createdActors)
+    for (i of createdActors)
         i.destroy();
 
     resetState();
